/******************************************************************************
 *                                                                            *
 *                                Copyright by                                *
 *                                                                            *
 *                              Azoteq (Pty) Ltd                              *
 *                          Republic of South Africa                          *
 *                                                                            *
 *                           Tel: +27(0)21 863 0033                           *
 *                           E-mail: info@azoteq.com                          *
 *                                                                            *
 * ========================================================================== *
 * @file        iqs7222d-example-code.ino                                     *
 * @brief       IQS7222D Squaretangle Trackpad Example code                   *
 *              (PCB Version - AZP1196A3)                                     *
 *              This example project demonstrates how to write the desired    *
 *              settings to the IQS7222D in order to use the IQS7222D EV-Kit  *
 *              with a single trackpad and three programmable feedback        *
 *              indicators(TOUT0, TOUT1 and TOUT2).                           *
 *                                                                            *
 *              All data is displayed over serial communication with the      *
 *              following outputs:                                            *
 *                  - Finger Position Feedback (X and Y)                      *
 *                  - Power Mode Switching (If Enabled)                       *
 *                  - Forced Communication                                    *
 *                  - Software Reset                                          *
 * @author      Azoteq PTY Ltd                                                *
 * @version     v1.1                                                          *
 * @date        2023-07-14                                                    *
 ******************************************************************************/

#include <Arduino.h>
#include "src\IQS7222D\IQS7222D.h"

/*** Defines ***/
#define DEMO_IQS7222D_ADDR        0x48
#define DEMO_IQS7222D_POWER_PIN   4
#define DEMO_IQS7222D_RDY_PIN     7  // Important: Must be a pin that supports interrupts

/*** Instances ***/
IQS7222D iqs7222d;

/*** Global Running Variables ***/
iqs7222d_power_modes running_power_mode = IQS7222D_NORMAL_POWER;
uint16_t running_event_flags = 0;
uint16_t running_x_output = 65535;
uint16_t running_y_output = 65535;
iqs7222d_gestures_e running_gestures = IQS7222D_GESTURE_NONE;

bool show_data = false;

void setup()
{
   /* Start Serial Communication */
  Serial.begin(115200);
  while(!Serial);
  Serial.println("Start Serial communication");

  /* Power On IQS7222D */
  pinMode(DEMO_IQS7222D_POWER_PIN, OUTPUT);
  delay(200);
  digitalWrite(DEMO_IQS7222D_POWER_PIN, LOW);
  delay(200);
  digitalWrite(DEMO_IQS7222D_POWER_PIN, HIGH);

  /* Initialize the IQS7222D with input parameters device address and RDY pin */
  iqs7222d.begin(DEMO_IQS7222D_ADDR, DEMO_IQS7222D_RDY_PIN);
  Serial.println("IQS7222D Ready");
  delay(1);
}

/* Main run loop */
void loop()
{
  /* Read new data from IQS7222D if available  (RDY Line Low) */
  iqs7222d.run();

  /* Function to initialize a force communication window. */
  force_comms_and_reset();

  /* If data was updated, display data read from IQS7222D */
  if (iqs7222d.new_data_available)
  {
    /* Print the following if the new data did not come from a force comms command */
    if (!show_data)
    {
      if(printData())
      {
        printGesture();
        printCoordinates();
        printPowerMode();
      }
    }
    /* Display the heading when a finger is lifted from the trackpad */
    printHeading();
    /* Set this flag to false to indicate that the new data was already displayed/used */
    iqs7222d.new_data_available = false;
    /* Check if a force comms command was sent and if we should display the data read */
    show_iqs7222d_data();
  }
}

/* Check if one of the gesture flags is set and display which one */
void printGesture(void)
{
  iqs7222d_gestures_e tempGestures = iqs7222d.getGestures();

  switch (tempGestures)
  {
    case IQS7222D_GESTURE_TAP:
      Serial.print("Tap\t\t");
      break;

    case IQS7222D_GESTURE_FLICK:
      Serial.print("Flick\t\t");
      break;

    case IQS7222D_GESTURE_SWIPE_X_POS:
      Serial.print("Swipe X +\t");
      break;

    case IQS7222D_GESTURE_SWIPE_Y_POS:
      Serial.print("Swipe Y +\t");
      break;

    case IQS7222D_GESTURE_SWIPE_X_NEG:
      Serial.print("Swipe X -\t");
      break;

    case IQS7222D_GESTURE_SWIPE_Y_NEG:
      Serial.print("Swipe Y -\t");
      break;

    default:
      Serial.print("-\t\t");
      break;
  }

  /* Update the running values for the next print heading call */
  running_gestures = tempGestures;
}

/* - Determines and prints current power mode
 * - Updates the running variable
 */
void printPowerMode(void)
{
 iqs7222d_power_modes tempPowerMode = iqs7222d.getPowerMode();

  switch (tempPowerMode)
  {
    case IQS7222D_NORMAL_POWER:
      Serial.println("Normal");
      break;

    case IQS7222D_LOW_POWER:
      Serial.println("Low");
      break;
      
    case IQS7222D_ULP:
      Serial.println("Ultra-Low");
      break;
  }

  /* Update the running values for the next print heading call */
  running_power_mode = tempPowerMode;
}

/* Function to print the heading on the release of a previous event:
 * - A coordinate of 65535 is equivalent to no touch and
 * - A touch event was reported by IC
 */

void printHeading(void)
{
  /* Check if a touch event occurred and the finger is no longer on the trackpad */
  if(
      (iqs7222d.getTrackPadCoordinate(IQS7222D_X_AXIS) == 0xFFFF)
      &&  (iqs7222d.getTrackPadCoordinate(IQS7222D_Y_AXIS) == 0xFFFF)
      && (iqs7222d.getTouchEvent() == true)  
    )
    {
      Serial.println("\nGesture:\tFinger 1 X:\tFinger 1 Y:\tPower Mode:");
    }
}

/* Function to determine if it is necessary to print all the relevant data 
in the serial terminal */
bool printData(void)
{
  /* See if it is necessary to display the coordinates, power mode, or gestures */
  if(
      (iqs7222d.getTrackPadCoordinate(IQS7222D_X_AXIS) != running_x_output)
      || (iqs7222d.getTrackPadCoordinate(IQS7222D_Y_AXIS) != running_y_output)
      || (iqs7222d.getPowerMode() != running_power_mode)
      || iqs7222d.getTrackpadEvent()
    )
  {
    return true;
  }
  else
  {
    return false;
  }
}

/* Function to return X and Y coordinates of the trackpad */
void printCoordinates(void)
{
  uint16_t buffer = iqs7222d.getTrackPadCoordinate(IQS7222D_X_AXIS);

  Serial.print(buffer);   // Print X coordinates
  Serial.print("\t\t");   // Spacing to match headings in Print

  running_x_output = buffer; /* Update the running values for the next print heading call */

  buffer = iqs7222d.getTrackPadCoordinate(IQS7222D_Y_AXIS);

  Serial.print(buffer);  // Print Y coordinates
  Serial.print("\t\t");  // Spacing to match headings in Print

  running_y_output = buffer; /* Update the running values for the next print heading call */
}

/* Force the IQS7222D to open a RDY window and read the current state of the device */
void force_comms_and_reset(void)
{
  char message = read_message();

  /* If an 'f' was received over serial, open a forced communication window and
  print the new data received */
  if(message == 'f')
  {
    iqs7222d.force_I2C_communication(); // prompt the IQS7222D
    show_data = true;
  }

  /* If an 'r' was received over serial, request a software(SW) reset */
  if(message == 'r')
  {
    Serial.println("Software Reset Requested!");
    iqs7222d.force_I2C_communication(); // Request a RDY window
    iqs7222d.iqs7222d_state.state = IQS7222D_STATE_SW_RESET;
    running_power_mode = IQS7222D_NORMAL_POWER;
  }
}

/* Read a message sent over serial communication */
char read_message(void)
{
  while (Serial.available())
  {
    if (Serial.available() > 0)
    {
      return Serial.read();
    }
  }

  return '\n';
}

/* Shows the current IQS7222D data when a force comms command was sent */
void show_iqs7222d_data()
{
  if (show_data)
  {
    Serial.println("\n***********************************************************");
    Serial.println("************* IQS7222D DATA - FORCED COMMS ****************");
    Serial.println("***********************************************************");
    Serial.println("Gesture:\tFinger 1 X:\tFinger 1 Y:\tPower Mode:");
    printGesture();
    printCoordinates();
    /* Set power mode to unknown otherwise the mode won't be displayed if it
       was the same as previously */
    running_power_mode = IQS7222D_POWER_UNKNOWN;
    printPowerMode();
    Serial.println("***********************************************************");
    show_data = false;
  }
}
