/**
 ******************************************************************************
 * @file     IQS7222D.cpp
 * @brief    This file contains the constructors and methods which allow ease
 *           of use of an IQS7222D capacitive touch controller with configurable
 *           output pins and low-power options. This class provides an easy
 *           means of initializing and interacting with the IQS7222D device
 *           from an Arduino.
 * @author   JN. Lochner
 * @version  V1.1
 * @date     2023
 ******************************************************************************
 * @attention  Makes use of the following standard Arduino libraries:
 * - Arduino.h -> Included in IQS7222D.h, comes standard with Arduino
 * - Wire.h    -> Included in IQS7222D.h, comes standard with Arduino
 *****************************************************************************/

/* Include Files */
#include "IQS7222D.h"

#if IQS7222D_IC_VERSION == 0
#include "IQS7222D_init_IC_version_1.2.h"
#elif IQS7222D_IC_VERSION == 1
#include "IQS7222D_init_IC_version_1.1.h"
#endif

/* Private Global Variables */
bool iqs7222d_deviceRDY = false;
uint8_t iqs7222d_ready_pin;

/* Private Functions */
void iqs7222d_ready_interrupt(void);

/*****************************************************************************/
/*                              CONSTRUCTORS                                 */
/*****************************************************************************/
IQS7222D::IQS7222D()
{
}

/*****************************************************************************/
/*                             PUBLIC METHODS                                */
/*****************************************************************************/

/**
 * @name   begin
 * @brief  A method to initialize the IQS7222D device with the device address
 *         and ready pin specified by the user.
 * @param  deviceAddressIn -> The address of the IQS7222D device.
 * @param  readyPinIn      -> The Arduino pin which is connected to the RDY/MCLR
 *                            pin of the IQS7222D device.
 * @retval None.
 * @note   -  Receiving a true return value does not mean that initialization
 *            was successful.
 *         -  Receiving a true return value only means that the IQS device responded
 *            to the request for communication.
 *         -  Receiving a false return value means that initialization did not take
 *            place at all. If communication is successfully established then it
 *            is unlikely that initialization will fail.
 */
void IQS7222D::begin(uint8_t deviceAddressIn, uint8_t readyPinIn)
{
  /* Initialize I2C communication here, since this library can't function without it. */
  Wire.begin();
  Wire.setClock(400000);

  _deviceAddress = deviceAddressIn;
  iqs7222d_ready_pin = readyPinIn;
  attachInterrupt(digitalPinToInterrupt(iqs7222d_ready_pin), iqs7222d_ready_interrupt, CHANGE);

  /* Initialize "running" and "init" state machine variables. */
  iqs7222d_state.state      = IQS7222D_STATE_START;
  iqs7222d_state.init_state = IQS7222D_INIT_VERIFY_PRODUCT;
}

/**
 * @name   init
 * @brief  A method that runs through a normal start-up routine to set up the
 *         IQS7222D with the desired settings from the IQS7222D_init.h file.
 * @retval Returns true if the full start-up routine has been completed,
 *         returns false if not.
 * @param  None.
 * @note   No false return will be given, the program will thus be stuck when
 *         one of the cases is not able to finish. See serial communication to
 *         find the ERROR case. Note that the Serial.println commands can be
 *         commented out to save on dynamic memory.
 */
bool IQS7222D::init(void)
{
  uint16_t prod_num;
  uint8_t ver_maj, ver_min;

  switch (iqs7222d_state.init_state)
  {
    /* Verifies product number to determine if the correct device is connected for this example */
    case IQS7222D_INIT_VERIFY_PRODUCT:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_VERIFY_PRODUCT");
        prod_num  = getProductNum(RESTART);
        ver_maj   = getMajorVersion(RESTART);
        ver_min   = getMinorVersion(STOP);
        Serial.print("\t\tProduct number is: ");
        Serial.print(prod_num);
        Serial.print(" v");
        Serial.print(ver_maj);
        Serial.print(".");
        Serial.println(ver_min);
        if(prod_num == IQS7222D_PRODUCT_NUM)
        {
          Serial.println("\t\tIQS7222D Release UI Confirmed!");
          iqs7222d_state.init_state = IQS7222D_INIT_READ_RESET;
        }
        else
        {
          Serial.println("\t\tDevice is not a IQS7222D!");
          iqs7222d_state.init_state = IQS7222D_INIT_NONE;
        }
      }
    break;

    /* Verify if a reset has occurred */
    case IQS7222D_INIT_READ_RESET:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_READ_RESET");
        updateInfoFlags(RESTART);
        if (checkReset())
        {
          Serial.println("\t\tReset event occurred.");
          iqs7222d_state.init_state = IQS7222D_INIT_UPDATE_SETTINGS;
        }
        else
        {
          Serial.println("\t\t No Reset Event Detected - Request SW Reset");
          iqs7222d_state.init_state = IQS7222D_INIT_CHIP_RESET;
        }
      }
    break;

    /* Perform SW Reset */
    case IQS7222D_INIT_CHIP_RESET:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_CHIP_RESET");

        //Perform SW Reset
        SW_Reset(STOP);
        Serial.println("\t\tSoftware Reset Bit Set.");
        delay(100);
        iqs7222d_state.init_state = IQS7222D_INIT_READ_RESET;
      }
    break;

    /* Write all settings to IQS7222D from .h file */
    case IQS7222D_INIT_UPDATE_SETTINGS:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_UPDATE_SETTINGS");
        writeMM(RESTART);
        iqs7222d_state.init_state = IQS7222D_INIT_ACK_RESET;
      }
    break;

    /* Acknowledge that the device went through a reset */
    case IQS7222D_INIT_ACK_RESET:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_ACK_RESET");
        acknowledgeReset(STOP);
        iqs7222d_state.init_state = IQS7222D_INIT_ATI;
      }
      break;

    /* Run the ATI algorithm to recalibrate the device with newly added settings */
    case IQS7222D_INIT_ATI:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_ATI");
        ReATI(STOP);
        iqs7222d_state.init_state = IQS7222D_INIT_WAIT_FOR_ATI;
        Serial.println("\tIQS7222D_INIT_WAIT_FOR_ATI");
      }
    break;

    /* Read the ATI Active bit to see if the rest of the program can continue */
    case IQS7222D_INIT_WAIT_FOR_ATI:
      if(iqs7222d_deviceRDY)
      {
        if(!readATIactive())
        {
          Serial.println("\t\tDONE");
          iqs7222d_state.init_state = IQS7222D_INIT_READ_DATA;
        }
      }
    break;

    /* Read the latest data from the iqs7222d */
    case IQS7222D_INIT_READ_DATA:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_READ_DATA");
        queueValueUpdates();
        iqs7222d_state.init_state = IQS7222D_INIT_ACTIVATE_STREAMINTOUCH_MODE;
      }
    break;

    /* Turn on I2C Event mode */
    case IQS7222D_INIT_ACTIVATE_EVENT_MODE:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_ACTIVATE_EVENT_MODE");
        setEventMode(STOP);
        iqs7222d_state.init_state = IQS7222D_INIT_DONE;
      }
    break;

    /* Turn on I2C Stream mode */
    case IQS7222D_INIT_ACTIVATE_STREAMINTOUCH_MODE:
      if(iqs7222d_deviceRDY)
      {
        Serial.println("\tIQS7222D_INIT_ACTIVATE_STREAMINTOUCH_MODE");
        setStreamInTouchMode(STOP);
        iqs7222d_state.init_state = IQS7222D_INIT_DONE;
      }
    break;

    /* If all operations have been completed correctly, the RDY pin can be set
     * up as an interrupt to indicate when new data is available */
    case IQS7222D_INIT_DONE:
      Serial.println("\tIQS7222D_INIT_DONE");
      new_data_available = true;
      return true;
    break;

    default:
      break;
  }
  return false;
}

/**
 * @name   run
 * @brief  Run the device ready check and read the necessary bytes when the
 *         IQS7222D has pulled the RDY line low. The new_data_available flag
 *         will be set when a ready low is received from the IQS7222D.
 * @param  None
 * @retval None.
 * @note   queueValueUpdates can be edited by the user if other data should be
 *         read every time a RDY window is received.
 */
void IQS7222D::run(void)
{
  switch (iqs7222d_state.state)
  {
    /* After a hardware reset, this is the starting position of the main state machine */
    case IQS7222D_STATE_START:
      Serial.println("IQS7222D Initialization:");
      iqs7222d_state.state = IQS7222D_STATE_INIT;
    break;

    /* Perform the initialization routine on the IQS7222D */
    case IQS7222D_STATE_INIT:
      if(init())
      {
        Serial.println("IQS7222D Initialization complete!\n");
        Serial.println("Gesture:\tFinger 1 X:\tFinger 1 Y:\tPower Mode:");
        iqs7222d_state.state = IQS7222D_STATE_RUN;
      }
    break;

    /* Send an 12C software reset in the next rdy window */
    case IQS7222D_STATE_SW_RESET:
      if(iqs7222d_deviceRDY)
      {
        SW_Reset(STOP);
        iqs7222d_state.state = IQS7222D_STATE_RUN;
      }
    break;

    /* Continuous reset monitoring state, ensure no reset event has occurred
       for data to be valid */
    case IQS7222D_STATE_CHECK_RESET:
      if(checkReset())
      {
        Serial.println("Reset occurred!\n");
        new_data_available = false;
        iqs7222d_state.state = IQS7222D_STATE_START;
        iqs7222d_state.init_state = IQS7222D_INIT_VERIFY_PRODUCT;
      }
      /* A reset did not occur, move to the run state and wait for a new ready window */
      else
      {
        new_data_available = true; /* No reset, thus data is valid */
        iqs7222d_state.state = IQS7222D_STATE_RUN;
      }
    break;

    /* If a RDY Window is open, read the latest values from the IQS7222D */
    case IQS7222D_STATE_RUN:
      if(iqs7222d_deviceRDY)
      {
        queueValueUpdates();
        iqs7222d_deviceRDY = false;
        new_data_available = false;
        iqs7222d_state.state = IQS7222D_STATE_CHECK_RESET;
      }
    break;
  }
}

/**
 * @name    iqs7222d_ready_interrupt
 * @brief   A method used as an interrupt function. Activated when a
 *          High -> Low (falling edge) and Low -> High (rising edge) interrupt
 *          is seen on the correct Arduino interrupt pin.
 * @param   None.
 * @retval  None.
 * @note    Keep this function as simple as possible to prevent stuck states
 *          and slow operations.
 */
void iqs7222d_ready_interrupt(void)
{
    if(digitalRead(iqs7222d_ready_pin))
  {
    iqs7222d_deviceRDY = false;
  }
  else
  {
    iqs7222d_deviceRDY = true;
  }
}

/**
  * @name   clearRDY
  * @brief  A method used to clear the ready interrupt bit.
  * @param  None.
  * @retval None.
  */
void IQS7222D::clearRDY(void)
{
  iqs7222d_deviceRDY = false;
}

/**
  * @name   getRDYStatus
  * @brief  A method used to retrieve the device RDY status.
  * @param  None.
  * @retval Returns the boolean IQS323 RDY state.
  *         - True when RDY line is LOW
  *         - False when RDY line is HIGH
  */
bool IQS7222D::getRDYStatus(void)
{
  return iqs7222d_deviceRDY;
}

/**
 * @name    queueValueUpdates
 * @brief   All I2C read operations in the queueValueUpdates method will be
 *          performed each time the IQS7222D opens a RDY window.
 * @param   None.
 * @retval  None.
 * @note    Any Address in the memory map can be read from here. This is where
 *          data read from the chip gets updated.
 */
void IQS7222D::queueValueUpdates(void)
{
  uint8_t transferBytes[14];	// The array which will hold the bytes to be transferred.

  /* Read the system status bytes 0x10 - 0x16. */
  readRandomBytes(IQS7222D_MM_SYSTEM_STATUS, 14, transferBytes, STOP);

  /* Assign the System Status */
  IQSMemoryMap.SYSTEM_STATUS[0] =  transferBytes[0];
  IQSMemoryMap.SYSTEM_STATUS[1] =  transferBytes[1];

  /* Assign the Events */
  IQSMemoryMap.EVENTS[0]  = transferBytes[2];
  IQSMemoryMap.EVENTS[1]  = transferBytes[3];

  /* Assign the Proximity event states */
  IQSMemoryMap.PROX_EVENT_STATES[0]  = transferBytes[4];
  IQSMemoryMap.PROX_EVENT_STATES[1] = transferBytes[5];

  /* Assign the Touch event states */
  IQSMemoryMap.TOUCH_EVENT_STATES[0]  = transferBytes[6];
  IQSMemoryMap.TOUCH_EVENT_STATES[1]  = transferBytes[7];

  /* Assign the Trackpad X coordinates */
  IQSMemoryMap.TRACKPAD_X_OUTPUT[0]  = transferBytes[8];
  IQSMemoryMap.TRACKPAD_X_OUTPUT[1]  = transferBytes[9];

  /* Assign the Trackpad Y coordinates */
  IQSMemoryMap.TRACKPAD_Y_OUTPUT[0]  = transferBytes[10];
  IQSMemoryMap.TRACKPAD_Y_OUTPUT[1]  = transferBytes[11];

  /* Assign the Trackpad event flags */
  IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0]  = transferBytes[12];
  IQSMemoryMap.TRACKPAD_EVENT_FLAGS[1]  = transferBytes[13];
}

/**
  * @name	  readATIactive
  * @brief  A method that checks if the ATI routine is still active
  * @param  None.
  * @retval Returns true if the ATI_ACTIVE_BIT is cleared, false if the
  *         ATI_ACTIVE_BIT is set.
  * @note   If the ATI routine is active the channel states (NONE, PROX, TOUCH)
  *         might exhibit unwanted behaviour. Thus it is advised to wait for
  *         the routine to complete before requesting data.
  */
bool IQS7222D::readATIactive(void)
{
  /* Read the latest info flags from IQS7222D*/
  updateInfoFlags(STOP);

  /* Return the ATI Active status */
  return getBit(IQSMemoryMap.SYSTEM_STATUS[0], IQS7222D_ATI_ACTIVE_BIT);
}

/**
 * @name	  checkReset
 * @brief   A method that checks if the device has reset and returns the reset
 *          status.
 * @param   None.
 * @retval  Returns true if a reset has occurred, false if no reset has occurred.
 * @note    If a reset has occurred the device settings should be reloaded using
 *          the begin function. After new device settings have been reloaded the
 *          acknowledge reset function can be used to clear the reset flag.
 */
bool IQS7222D::checkReset(void)
{
	/* Perform a bitwise AND operation with the DEVICE_RESET_BIT to return the
     reset status */
  return getBit(IQSMemoryMap.SYSTEM_STATUS[0], IQS7222D_DEVICE_RESET_BIT);
}

/**
  * @name	  checkProductNum
  * @brief  A method that checks the device product number.
  * @param  stopOrRestart -> Specifies whether the communications window must be
  *                          kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns the product number as a 16-bit unsigned integer value.
  * @note   If the product is not correctly identified an appropriate messages
  *         should be displayed.
  */
uint16_t IQS7222D::getProductNum(bool stopOrRestart)
{
	uint8_t transferBytes[2];	      // A temporary array to hold the byte to be transferred.
  uint8_t prodNumLow = 0;         // Temporary storage for the counts low byte.
  uint8_t prodNumHigh = 0;        // Temporary storage for the counts high byte.
  uint16_t prodNumReturn = 0;     // The 16-bit return value.

	/* Read the Device info from the IQS7222D. */
	readRandomBytes(IQS7222D_MM_APP_VERSION_INFO, 2, transferBytes, stopOrRestart);

  /* Construct the 16-bit return value. */
  prodNumLow = transferBytes[0];
  prodNumHigh = transferBytes[1];
  prodNumReturn = (uint16_t)(prodNumLow);
  prodNumReturn |= (uint16_t)(prodNumHigh<<8);
  /* Return the counts value. */
  return prodNumReturn;
}

/**
  * @name	  getMajorVersion
  * @brief  A method that checks the device firmware version's major value.
  * @param  stopOrRestart -> Specifies whether the communications window must be
  *                          kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns the major version number as an 8-bit unsigned integer value.
  */
uint8_t IQS7222D::getMajorVersion(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// A temporary array to hold the byte to be transferred.
  uint8_t ver_maj = 0;      // Temporary storage for the firmware version major number.

	/* Read the Device info from the IQS7222D. */
	readRandomBytes(IQS7222D_MM_APP_VERSION_INFO_1, 2, transferBytes, stopOrRestart);

  /* Get major value from correct byte */
  ver_maj = transferBytes[0];
  /* Return the major firmware version number value. */
  return ver_maj;
}

/**
  * @name	  getMinorVersion
  * @brief  A method that checks the device firmware version's minor value.
  * @param  stopOrRestart -> Specifies whether the communications window must be
  *                          kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns the minor version number as an 8-bit unsigned integer value.
  */
uint8_t IQS7222D::getMinorVersion(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// A temporary array to hold the byte to be transferred.
  uint8_t ver_min = 0;         // Temporary storage for the firmware version minor number.

	/* Read the Device info from the IQS7222D. */
	readRandomBytes(IQS7222D_MM_APP_VERSION_INFO_2, 2, transferBytes, stopOrRestart);
  /* get major value from correct byte */
  ver_min = transferBytes[0];
  /* Return the minor firmware version number value. */
  return ver_min;
}

/**
 * @name	  acknowledgeReset
 * @brief   A method that clears the Show Reset bit by setting the Ack Reset bit
 * @param   stopOrRestart -> Specifies whether the communications window must be
 *                          kept open or must be closed after this action.
 *                          Use the STOP and RESTART definitions.
 * @retval  None.
 * @note    If a reset has occurred the device settings should be reloaded using
 *          the begin function. After new device settings have been reloaded
 *          this method should be used to clear the DEVICE_RESET bit.
 */
void IQS7222D::acknowledgeReset(bool stopOrRestart)
{
	uint8_t transferByte[2];	// Array to store the bytes transferred.
	/* Read the System Flags from the IQS7222D, these must be read first in order
     not to change any settings. */
	readRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, RESTART);
	/* Write the ACK_RESET_BIT to clear the Reset Event Flag. */
	transferByte[0] = setBit(transferByte[0], IQS7222D_ACK_RESET_BIT);
	/* Write the new byte to the System Flags address. */
	writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, stopOrRestart);}

/**
 * @name   ReATI
 * @brief  A method that sets the TP ReATI bit to force the IQS7222D
 *         device to run the Automatic Tuning Implementation (ATI) routine on
 *         the enabled channels.
 * @param  stopOrRestart -> Specifies whether the communications window must be
 *                          kept open or must be closed after this action.
 *                          Use the STOP and RESTART definitions.
 * @retval None.
 * @note   To force ATI, bit 5 in the System Control register is set.
 */
void IQS7222D::ReATI(bool stopOrRestart)
{
  uint8_t transferByte[2]; // Array to store the bytes transferred.

  readRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, RESTART);
  /* Set the RE_ATI_BIT in the CONTROL_SETTINGS */
  transferByte[0] = setBit(transferByte[0], IQS7222D_RE_ATI_BIT);
  /* Write the new byte to the required device. */
  writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, stopOrRestart);
}

/**
 * @name   SW_Reset
 * @brief  A method that sets the SW RESET bit to force the IQS7222D
 *         device to do a SW reset.
 * @param  stopOrRestart -> Specifies whether the communications window must be
 *                          kept open or must be closed after this action.
 *                          Use the STOP and RESTART definitions.
 * @retval None.
 * @note   To perform SW Reset, bit 9 in SYSTEM_CONTROL is set.
 */
void IQS7222D::SW_Reset(bool stopOrRestart)
{
  uint8_t transferByte[2]; // Array to store the bytes transferred.

  readRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, RESTART);
  /* Set the SW_RESET_BIT in the CONTROL_SETTINGS register */
  transferByte[0] = setBit(transferByte[0], IQS7222D_SW_RESET_BIT);
  /* Write the new byte to the required device. */
  writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, stopOrRestart);
}

/**
  * @name   setStreamMode
  * @brief  A method to set the IQS7211A device into streaming mode.
  * @param  stopOrRestart -> Specifies whether the communications window must be
  *                          kept open or must be closed after this action.
  *              Use the STOP and RESTART definitions.
  * @retval None.
  * @note   All other bits at the register address are preserved.
  */
void IQS7222D::setStreamMode(bool stopOrRestart)
{
  uint8_t transferByte[2]; // The array which will hold the bytes which are transferred.

  /* First read the bytes at the memory address so that they can be preserved */
  readRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, RESTART);
  /* Clear the Interface Selection Bits in the CONTROL_SETTINGS register */
  transferByte[0] = clearBit(transferByte[0], IQS7222D_EVENT_MODE_BIT_0);
  transferByte[0] = clearBit(transferByte[0], IQS7222D_EVENT_MODE_BIT_1);
  /* Write the bytes back to the device */
  writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, stopOrRestart);
}

/**
 * @name   setEventMode
 * @brief  A method to set the IQS7222D device into event mode.
 * @param  stopOrRestart -> Specifies whether the communications window must be
 *                          kept open or must be closed after this action.
 *                          Use the STOP and RESTART definitions.
 * @retval None.
 * @note   All other bits at the register address are preserved.
 */
void IQS7222D::setEventMode(bool stopOrRestart)
{
  uint8_t transferByte[2]; // The array which will hold the bytes which are transferred.

  /* First read the bytes at the memory address so that they can be preserved */
  readRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, RESTART);
  /* Set/Clear the Interface Selection Bits in the CONTROL_SETTINGS register */
  transferByte[0] = setBit(transferByte[0], IQS7222D_EVENT_MODE_BIT_0);
  transferByte[0] = clearBit(transferByte[0], IQS7222D_EVENT_MODE_BIT_1);
  /* Write the bytes back to the device */
  writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, stopOrRestart);
}

/**
 * @name   setStreamInTouchMode
 * @brief  A method to set the IQS7222D device into Stream in touch mode.
 * @param  stopOrRestart -> Specifies whether the communications window must be
 *                          kept open or must be closed after this action.
 *                          Use the STOP and RESTART definitions.
 * @retval None.
 * @note   All other bits at the register address are preserved.
 */
void IQS7222D::setStreamInTouchMode(bool stopOrRestart)
{
  uint8_t transferByte[2]; // The array which will hold the bytes which are transferred.

  /* First read the bytes at the memory address so that they can be preserved */
  readRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, RESTART);
  /* Set/Clear the Interface Selection Bits in the CONTROL_SETTINGS register */
  transferByte[0] = clearBit(transferByte[0], IQS7222D_EVENT_MODE_BIT_0);
  transferByte[0] = setBit(transferByte[0], IQS7222D_EVENT_MODE_BIT_1);
  /* Write the bytes back to the device */
  writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 2, transferByte, stopOrRestart);
}

/**
 * @name   updateInfoFlags
 * @brief  A method which reads the IQS7222D info flags and assigns them to the
 *         SYSTEM_STATUS register.
 * @param  stopOrRestart -> Specifies whether the communications window must be
 *                          kept open or must be closed after this action.
 *              			      Use the STOP and RESTART definitions.
 * @retval None.
 */
void IQS7222D::updateInfoFlags(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// The array which will hold the bytes to be transferred.

	/* Read the info flags. */
	readRandomBytes(IQS7222D_MM_SYSTEM_STATUS, 2, transferBytes, stopOrRestart);
	/* Assign the info flags to the info flags memory map register. */
  IQSMemoryMap.SYSTEM_STATUS[0] =  transferBytes[0];
  IQSMemoryMap.SYSTEM_STATUS[1] =  transferBytes[1];
}

/**
  * @name   getTrackPadCoordinate
  * @brief  A method that reads the current position of a finger
  * @param  TrackPadAxisSelect -> The axis you want the coordinates of.
  * @retval Returns a 16-bit unsigned integer value that contains the finger
  *         coordinates of a specific axis from 0 to the resolution maximum.
  * @note   See the iqs7222d_trackpad_axis_e typedef for the axis options.
  */
uint16_t IQS7222D::getTrackPadCoordinate(iqs7222d_trackpad_axis_e TrackPadAxisSelect)
{
  uint16_t buffer;

  // Create the 16-bit value to be returned
  switch(TrackPadAxisSelect)
  {
    case IQS7222D_X_AXIS:
    {
        buffer = IQSMemoryMap.TRACKPAD_X_OUTPUT[0];
        buffer += IQSMemoryMap.TRACKPAD_X_OUTPUT[1]<<8;
        return buffer;
    }
    break;

    case IQS7222D_Y_AXIS:
    {
        buffer = IQSMemoryMap.TRACKPAD_Y_OUTPUT[0];
        buffer += IQSMemoryMap.TRACKPAD_Y_OUTPUT[1]<<8;
        return buffer;
    }
    break;

    default:
      return 0xFFFF;
    break;
  }
}

/**
  * @name   getGestures
  * @brief  A method which reads the TRACKPAD_EVENT_FLAGS memory map register
  *         and returns the specific gesture that caused an event.
  * @param  void
  * @retval Returns the gesture type that occurred on the IQS7222D.
  * @note   See the iqs7222d_gestures_e typedef for all possible gestures.
  */
iqs7222d_gestures_e IQS7222D::getGestures(void)
{
  if (getBit(IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0], IQS7222D_GESTURE_TAP_BIT))
  {
    return IQS7222D_GESTURE_TAP;
  }
  else if (getBit(IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0], IQS7222D_GESTURE_FLICK_BIT))
  {
    return IQS7222D_GESTURE_FLICK;
  }
  else if (getBit(IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0], IQS7222D_GESTURE_SWIPE_X_POS_BIT))
  {
    return IQS7222D_GESTURE_SWIPE_X_POS;
  }
  else if (getBit(IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0], IQS7222D_GESTURE_SWIPE_Y_POS_BIT))
  {
    return IQS7222D_GESTURE_SWIPE_Y_POS;
  }
  else if (getBit(IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0], IQS7222D_GESTURE_SWIPE_X_NEG_BIT))
  {
    return IQS7222D_GESTURE_SWIPE_X_NEG;
  }
  else if (getBit(IQSMemoryMap.TRACKPAD_EVENT_FLAGS[0], IQS7222D_GESTURE_SWIPE_Y_NEG_BIT))
  {
    return IQS7222D_GESTURE_SWIPE_Y_NEG;
  }
  else
  {
    return IQS7222D_GESTURE_NONE;
  }
}

/**
  * @name   getTrackpadEvent
  * @brief  A method that reads the IQS7222D_TRACKPAD_EVENT_BIT in the 
  *         EVENTS memory map register.
  * @param  void
  * @retval Returns true if a trackpad event occurred, false if not.
  */
bool IQS7222D::getTrackpadEvent(void)
{
  return getBit(IQSMemoryMap.EVENTS[1], IQS7222D_TRACKPAD_EVENT_BIT);
}

/**
  * @name   getPowerMode
  * @brief  A method that reads the system flags register and returns the
  *         current power mode.
  * @param  None.
  * @retval Returns the current iqs7222d_power_modes state the device is in.
  * @note   See Datasheet on power mode options and timeouts.
  *         Normal Power, Low Power, and Ultra Low Power (ULP).
  */
iqs7222d_power_modes IQS7222D::getPowerMode(void)
{
  uint8_t buffer = getBit(IQSMemoryMap.SYSTEM_STATUS[0], IQS7222D_POWER_EVENT_BIT_0);
  buffer += getBit(IQSMemoryMap.SYSTEM_STATUS[0], IQS7222D_POWER_EVENT_BIT_1) << 1;

  if(buffer == IQS7222D_NORMAL_POWER_BIT)
  {
    return IQS7222D_NORMAL_POWER;
  }
  else if(buffer == IQS7222D_LOW_POWER_BIT)
  {
    return IQS7222D_LOW_POWER;
  }
  else if(buffer == IQS7222D_ULP_BIT)
  {
    return IQS7222D_ULP;
  }
  else
  {
    return IQS7222D_POWER_UNKNOWN;
  }
}

/**
  * @name   channel_touchState
  * @brief  A method which reads the TOUCH_EVENT_STATES register for a given
  *         channel and determines if the channel is in a touch state.
  * @param  channel -> The channel name on the IQS7222D (CH0-CH13) for which
  *                    a touch state needs to be determined.
  * @retval Returns true if a touch is active and false if there is no touch.
  * @note   See the iqs7222d_channel_e typedef for all possible channel names.
  */
bool IQS7222D::channel_touchState(iqs7222d_channel_e channel)
{
  switch(channel)
	{
		case IQS7222D_CH0:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH0_TOUCH_BIT);
		break;

    case IQS7222D_CH1:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH1_TOUCH_BIT);
		break;

    case IQS7222D_CH2:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH2_TOUCH_BIT);
		break;

    case IQS7222D_CH3:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH3_TOUCH_BIT);
		break;

    case IQS7222D_CH4:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH4_TOUCH_BIT);
		break;

    case IQS7222D_CH5:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH5_TOUCH_BIT);
		break;

    case IQS7222D_CH6:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH6_TOUCH_BIT);
		break;

    case IQS7222D_CH7:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[0], IQS7222D_CH7_TOUCH_BIT);
		break;

    case IQS7222D_CH8:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[1], IQS7222D_CH8_TOUCH_BIT);
		break;

    case IQS7222D_CH9:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[1], IQS7222D_CH9_TOUCH_BIT);
		break;

    case IQS7222D_CH10:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[1], IQS7222D_CH10_TOUCH_BIT);
		break;

    case IQS7222D_CH11:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[1], IQS7222D_CH11_TOUCH_BIT);
		break;

    case IQS7222D_CH12:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[1], IQS7222D_CH12_TOUCH_BIT);
		break;

    case IQS7222D_CH13:
			return getBit(IQSMemoryMap.TOUCH_EVENT_STATES[1], IQS7222D_CH13_TOUCH_BIT);
		break;

		default:
			return false;
		break;
	}
}

/**
  * @name   getTouchEvent
  * @brief  A method that reads the IQS7222D_TOUCH_EVENT_BIT in the 
  *         EVENTS memory map register.
  * @param  void
  * @retval Returns true if a touch event occurred, false if not.
  */
bool IQS7222D::getTouchEvent(void)
{
  return getBit(IQSMemoryMap.EVENTS[0], IQS7222D_TOUCH_EVENT_BIT);
}

/**
  * @name   channel_proxState
  * @brief  A method which reads the PROX_EVENT_STATES register for a given
  *         channel and determines if the channel is in a proximity state.
  * @param  channel -> The channel name on the IQS7222D (CH0-CH13) for which
  *         a proximity state needs to be determined.
  * @retval Returns true if proximity is active and false if there is no proximity.
  * @note   See the iqs7222d_channel_e typedef for all possible channel names.
  */
bool IQS7222D::channel_proxState(iqs7222d_channel_e channel)
{
  switch(channel)
	{
		case IQS7222D_CH0:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH0_PROX_BIT);
		break;

    case IQS7222D_CH1:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH1_PROX_BIT);
		break;

    case IQS7222D_CH2:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH2_PROX_BIT);
		break;

    case IQS7222D_CH3:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH3_PROX_BIT);
		break;

    case IQS7222D_CH4:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH4_PROX_BIT);
		break;

    case IQS7222D_CH5:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH5_PROX_BIT);
		break;

    case IQS7222D_CH6:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH6_PROX_BIT);
		break;

    case IQS7222D_CH7:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[0], IQS7222D_CH7_PROX_BIT);
		break;

    case IQS7222D_CH8:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[1], IQS7222D_CH8_PROX_BIT);
		break;

    case IQS7222D_CH9:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[1], IQS7222D_CH9_PROX_BIT);
		break;

    case IQS7222D_CH10:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[1], IQS7222D_CH10_PROX_BIT);
		break;

    case IQS7222D_CH11:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[1], IQS7222D_CH11_PROX_BIT);
		break;

    case IQS7222D_CH12:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[1], IQS7222D_CH12_PROX_BIT);
		break;

    case IQS7222D_CH13:
			return getBit(IQSMemoryMap.PROX_EVENT_STATES[1], IQS7222D_CH13_PROX_BIT);
		break;

		default:
			return false;
		break;
	}
}

/**
  * @name   getProxEvent
  * @brief  A method that reads the IQS7222D_PROX_EVENT_BIT in the 
  *         EVENTS memory map register.
  * @param  void
  * @retval Returns true if a prox event occurred, false if not.
  */
bool IQS7222D::getProxEvent(void)
{
  return getBit(IQSMemoryMap.EVENTS[0], IQS7222D_PROX_EVENT_BIT);
}

/*****************************************************************************/
/*								   			ADVANCED PUBLIC METHODS										         */
/*****************************************************************************/

/**
 * @name    writeMM
 * @brief   Function to write the whole memory map to the device (writable)
 *          registers
 * @param   stopOrRestart ->  Specifies whether the communications window must
 *                            be kept open or must be closed after this action.
 *                            Use the STOP and RESTART definitions.
 * @retval  None.
 * @note    IQS7222D_init.h -> exported GUI init.h file
 */
void IQS7222D::writeMM(bool stopOrRestart)
{

  uint8_t transferBytes[30]; // Temporary array which holds the bytes to be transferred.

//Converting Cycle 0 setup
transferBytes[0] = CYCLE_0_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_0_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_0_SETTINGS;
transferBytes[3] = CYCLE_0_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP0, 4, transferBytes, RESTART);

//Converting Cycle 1 setup
transferBytes[0] = CYCLE_1_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_1_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_1_SETTINGS;
transferBytes[3] = CYCLE_1_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP1, 4, transferBytes, RESTART);

//Converting Cycle 2 setup
transferBytes[0] = CYCLE_2_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_2_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_2_SETTINGS;
transferBytes[3] = CYCLE_2_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP2, 4, transferBytes, RESTART);

//Converting Cycle 3 setup
transferBytes[0] = CYCLE_3_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_3_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_3_SETTINGS;
transferBytes[3] = CYCLE_3_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP3, 4, transferBytes, RESTART);

//Converting Cycle 4 setup
transferBytes[0] = CYCLE_4_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_4_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_4_SETTINGS;
transferBytes[3] = CYCLE_4_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP4, 4, transferBytes, RESTART);

//Converting Cycle 5 setup
transferBytes[0] = CYCLE_5_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_5_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_5_SETTINGS;
transferBytes[3] = CYCLE_5_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP5, 4, transferBytes, RESTART);

//Converting Cycle 6 setup
transferBytes[0] = CYCLE_6_CONV_FREQ_FRAC;
transferBytes[1] = CYCLE_6_CONV_FREQ_PERIOD;
transferBytes[2] = CYCLE_6_SETTINGS;
transferBytes[3] = CYCLE_6_CTX_SELECT;
writeRandomBytes16(IQS7222D_CYCLE_SETUP6, 4, transferBytes, RESTART);

//Converting global cycle setup
transferBytes[0] = GLOBAL_CYCLE_SETUP_0;
transferBytes[1] = GLOBAL_CYCLE_SETUP_1;
transferBytes[2] = COARSE_DIVIDER_PRELOAD;
transferBytes[3] = FINE_DIVIDER_PRELOAD;
transferBytes[4] = COMPENSATION_PRELOAD_0;
transferBytes[5] = COMPENSATION_PRELOAD_1;
writeRandomBytes16(IQS7222D_GLOBAL_CYCLE_SETUP, 6, transferBytes, RESTART);
Serial.println("\t\t1. Write Cycle Settings");

//Converting Button Setup 0
transferBytes[0] = BUTTON_0_PROX_THRESHOLD;
transferBytes[1] = BUTTON_0_ENTER_EXIT;
transferBytes[2] = BUTTON_0_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_0_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_0_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_0_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP0, 6, transferBytes, RESTART);

//Converting Button Setup 1
transferBytes[0] = BUTTON_1_PROX_THRESHOLD;
transferBytes[1] = BUTTON_1_ENTER_EXIT;
transferBytes[2] = BUTTON_1_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_1_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_1_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_1_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP1, 6, transferBytes, RESTART);

//Converting Button Setup 2
transferBytes[0] = BUTTON_2_PROX_THRESHOLD;
transferBytes[1] = BUTTON_2_ENTER_EXIT;
transferBytes[2] = BUTTON_2_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_2_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_2_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_2_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP2, 6, transferBytes, RESTART);

//Converting Button Setup 3
transferBytes[0] = BUTTON_3_PROX_THRESHOLD;
transferBytes[1] = BUTTON_3_ENTER_EXIT;
transferBytes[2] = BUTTON_3_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_3_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_3_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_3_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP3, 6, transferBytes, RESTART);

//Converting Button Setup 4
transferBytes[0] = BUTTON_4_PROX_THRESHOLD;
transferBytes[1] = BUTTON_4_ENTER_EXIT;
transferBytes[2] = BUTTON_4_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_4_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_4_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_4_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP4, 6, transferBytes, RESTART);

//Converting Button Setup 5
transferBytes[0] = BUTTON_5_PROX_THRESHOLD;
transferBytes[1] = BUTTON_5_ENTER_EXIT;
transferBytes[2] = BUTTON_5_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_5_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_5_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_5_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP5, 6, transferBytes, RESTART);

//Converting Button Setup 6
transferBytes[0] = BUTTON_6_PROX_THRESHOLD;
transferBytes[1] = BUTTON_6_ENTER_EXIT;
transferBytes[2] = BUTTON_6_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_6_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_6_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_6_TOUCH_EVENT_TIMEOUT;
writeRandomBytes16(IQS7222D_BUTTON_SETUP6, 6, transferBytes, RESTART);

//Converting Button Setup 7
transferBytes[0] = BUTTON_7_PROX_THRESHOLD;
transferBytes[1] = BUTTON_7_ENTER_EXIT;
transferBytes[2] = BUTTON_7_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_7_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_7_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_7_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP7, 6, transferBytes, RESTART);

//Converting Button Setup 8
transferBytes[0] = BUTTON_8_PROX_THRESHOLD;
transferBytes[1] = BUTTON_8_ENTER_EXIT;
transferBytes[2] = BUTTON_8_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_8_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_8_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_8_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP8, 6, transferBytes, RESTART);


//Converting Button Setup 9
transferBytes[0] = BUTTON_9_PROX_THRESHOLD;
transferBytes[1] = BUTTON_9_ENTER_EXIT;
transferBytes[2] = BUTTON_9_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_9_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_9_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_9_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP9, 6, transferBytes, RESTART);

//Converting Button Setup 10
transferBytes[0] = BUTTON_10_PROX_THRESHOLD;
transferBytes[1] = BUTTON_10_ENTER_EXIT;
transferBytes[2] = BUTTON_10_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_10_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_10_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_10_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP10, 6, transferBytes, RESTART);

//Converting Button Setup 11
transferBytes[0] = BUTTON_11_PROX_THRESHOLD;
transferBytes[1] = BUTTON_11_ENTER_EXIT;
transferBytes[2] = BUTTON_11_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_11_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_11_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_11_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP11, 6, transferBytes, RESTART);

//Converting Button Setup 12
transferBytes[0] = BUTTON_12_PROX_THRESHOLD;
transferBytes[1] = BUTTON_12_ENTER_EXIT;
transferBytes[2] = BUTTON_12_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_12_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_12_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_12_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP12, 6, transferBytes, RESTART);

//Converting Button Setup 13
transferBytes[0] = BUTTON_13_PROX_THRESHOLD;
transferBytes[1] = BUTTON_13_ENTER_EXIT;
transferBytes[2] = BUTTON_13_TOUCH_THRESHOLD;
transferBytes[3] = BUTTON_13_TOUCH_HYSTERESIS;
transferBytes[4] = BUTTON_13_PROX_EVENT_TIMEOUT;
transferBytes[5] = BUTTON_13_TOUCH_EVENT_TIMEOUT;

writeRandomBytes16(IQS7222D_BUTTON_SETUP13, 6, transferBytes, RESTART);
Serial.println("\t\t2. Write Button Settings");

// Converting CH0 Setup
transferBytes[0] = CH0_SETUP_0;
transferBytes[1] = CH0_SETUP_1;
transferBytes[2] = CH0_ATI_SETTINGS_0;
transferBytes[3] = CH0_ATI_SETTINGS_1;
transferBytes[4] = CH0_MULTIPLIERS_0;
transferBytes[5] = CH0_MULTIPLIERS_1;
transferBytes[6] = CH0_ATI_COMPENSATION_0;
transferBytes[7] = CH0_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP, 8, transferBytes, RESTART);

// Converting CH1 Setup
transferBytes[0] = CH1_SETUP_0;
transferBytes[1] = CH1_SETUP_1;
transferBytes[2] = CH1_ATI_SETTINGS_0;
transferBytes[3] = CH1_ATI_SETTINGS_1;
transferBytes[4] = CH1_MULTIPLIERS_0;
transferBytes[5] = CH1_MULTIPLIERS_1;
transferBytes[6] = CH1_ATI_COMPENSATION_0;
transferBytes[7] = CH1_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL1, 8, transferBytes, RESTART);

// Converting CH2 Setup
transferBytes[0] = CH2_SETUP_0;
transferBytes[1] = CH2_SETUP_1;
transferBytes[2] = CH2_ATI_SETTINGS_0;
transferBytes[3] = CH2_ATI_SETTINGS_1;
transferBytes[4] = CH2_MULTIPLIERS_0;
transferBytes[5] = CH2_MULTIPLIERS_1;
transferBytes[6] = CH2_ATI_COMPENSATION_0;
transferBytes[7] = CH2_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL2, 8, transferBytes, RESTART);

// Converting CH3 Setup
transferBytes[0] = CH3_SETUP_0;
transferBytes[1] = CH3_SETUP_1;
transferBytes[2] = CH3_ATI_SETTINGS_0;
transferBytes[3] = CH3_ATI_SETTINGS_1;
transferBytes[4] = CH3_MULTIPLIERS_0;
transferBytes[5] = CH3_MULTIPLIERS_1;
transferBytes[6] = CH3_ATI_COMPENSATION_0;
transferBytes[7] = CH3_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL3, 8, transferBytes, RESTART);

// Converting CH4 Setup
transferBytes[0] = CH4_SETUP_0;
transferBytes[1] = CH4_SETUP_1;
transferBytes[2] = CH4_ATI_SETTINGS_0;
transferBytes[3] = CH4_ATI_SETTINGS_1;
transferBytes[4] = CH4_MULTIPLIERS_0;
transferBytes[5] = CH4_MULTIPLIERS_1;
transferBytes[6] = CH4_ATI_COMPENSATION_0;
transferBytes[7] = CH4_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL4, 8, transferBytes, RESTART);

//Changing CH5 Setup
transferBytes[0] = CH5_SETUP_0;
transferBytes[1] = CH5_SETUP_1;
transferBytes[2] = CH5_ATI_SETTINGS_0;
transferBytes[3] = CH5_ATI_SETTINGS_1;
transferBytes[4] = CH5_MULTIPLIERS_0;
transferBytes[5] = CH5_MULTIPLIERS_1;
transferBytes[6] = CH5_ATI_COMPENSATION_0;
transferBytes[7] = CH5_ATI_COMPENSATION_1;
writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL5, 8, transferBytes, RESTART);

//Changing CH6 Setup
transferBytes[0] = CH6_SETUP_0;
transferBytes[1] = CH6_SETUP_1;
transferBytes[2] = CH6_ATI_SETTINGS_0;
transferBytes[3] = CH6_ATI_SETTINGS_1;
transferBytes[4] = CH6_MULTIPLIERS_0;
transferBytes[5] = CH6_MULTIPLIERS_1;
transferBytes[6] = CH6_ATI_COMPENSATION_0;
transferBytes[7] = CH6_ATI_COMPENSATION_1;
writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL6, 8, transferBytes, RESTART);

//Changing CH7 Setup
transferBytes[0] = CH7_SETUP_0;
transferBytes[1] = CH7_SETUP_1;
transferBytes[2] = CH7_ATI_SETTINGS_0;
transferBytes[3] = CH7_ATI_SETTINGS_1;
transferBytes[4] = CH7_MULTIPLIERS_0;
transferBytes[5] = CH7_MULTIPLIERS_1;
transferBytes[6] = CH7_ATI_COMPENSATION_0;
transferBytes[7] = CH7_ATI_COMPENSATION_1;
writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL7, 8, transferBytes, RESTART);

//Changing CH8 Setup
transferBytes[0] = CH8_SETUP_0;
transferBytes[1] = CH8_SETUP_1;
transferBytes[2] = CH8_ATI_SETTINGS_0;
transferBytes[3] = CH8_ATI_SETTINGS_1;
transferBytes[4] = CH8_MULTIPLIERS_0;
transferBytes[5] = CH8_MULTIPLIERS_1;
transferBytes[6] = CH8_ATI_COMPENSATION_0;
transferBytes[7] = CH8_ATI_COMPENSATION_1;
writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL8, 8, transferBytes, RESTART);

// Converting CH9 Setup
transferBytes[0] = CH9_SETUP_0;
transferBytes[1] = CH9_SETUP_1;
transferBytes[2] = CH9_ATI_SETTINGS_0;
transferBytes[3] = CH9_ATI_SETTINGS_1;
transferBytes[4] = CH9_MULTIPLIERS_0;
transferBytes[5] = CH9_MULTIPLIERS_1;
transferBytes[6] = CH9_ATI_COMPENSATION_0;
transferBytes[7] = CH9_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL9, 8, transferBytes, RESTART);

// Converting CH10 Setup
transferBytes[0] = CH10_SETUP_0;
transferBytes[1] = CH10_SETUP_1;
transferBytes[2] = CH10_ATI_SETTINGS_0;
transferBytes[3] = CH10_ATI_SETTINGS_1;
transferBytes[4] = CH10_MULTIPLIERS_0;
transferBytes[5] = CH10_MULTIPLIERS_1;
transferBytes[6] = CH10_ATI_COMPENSATION_0;
transferBytes[7] = CH10_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL10, 8, transferBytes, RESTART);

// Converting CH11 Setup
transferBytes[0] = CH11_SETUP_0;
transferBytes[1] = CH11_SETUP_1;
transferBytes[2] = CH11_ATI_SETTINGS_0;
transferBytes[3] = CH11_ATI_SETTINGS_1;
transferBytes[4] = CH11_MULTIPLIERS_0;
transferBytes[5] = CH11_MULTIPLIERS_1;
transferBytes[6] = CH11_ATI_COMPENSATION_0;
transferBytes[7] = CH11_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL11, 8, transferBytes, RESTART);

// Converting CH12 Setup
transferBytes[0] = CH12_SETUP_0;
transferBytes[1] = CH12_SETUP_1;
transferBytes[2] = CH12_ATI_SETTINGS_0;
transferBytes[3] = CH12_ATI_SETTINGS_1;
transferBytes[4] = CH12_MULTIPLIERS_0;
transferBytes[5] = CH12_MULTIPLIERS_1;
transferBytes[6] = CH12_ATI_COMPENSATION_0;
transferBytes[7] = CH12_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL12, 8, transferBytes, RESTART);

//Converting CH13 Setup
transferBytes[0] = CH13_SETUP_0;
transferBytes[1] = CH13_SETUP_1;
transferBytes[2] = CH13_ATI_SETTINGS_0;
transferBytes[3] = CH13_ATI_SETTINGS_1;
transferBytes[4] = CH13_MULTIPLIERS_0;
transferBytes[5] = CH13_MULTIPLIERS_1;
transferBytes[6] = CH13_ATI_COMPENSATION_0;
transferBytes[7] = CH13_ATI_COMPENSATION_1;

writeRandomBytes16(IQS7222D_CRX_SELECT_GEN_SETUP_CHANNEL13, 8, transferBytes, RESTART);
Serial.println("\t\t3. Write Channel Setup Settings");

//Converting filter betas
transferBytes[0] = COUNTS_BETA_FILTER;
transferBytes[1] = LTA_BETA_FILTER;

writeRandomBytes16(IQS7222D_FILTER_BETA, 2, transferBytes, RESTART);

transferBytes[0] = LTA_FAST_BETA_FILTER;
transferBytes[1] = FILTER_RESERVED;
writeRandomBytes16(IQS7222D_FAST_FILTER_BETA, 2, transferBytes, RESTART);
Serial.println("\t\t4. Write Filter Betas");

//Converting trackpad general settings
transferBytes[0] = TRACKPAD_SETUP_GENERAL;
transferBytes[1] = TRACKPAD_FILTER;
transferBytes[2] = X_LOWER_CAL;
transferBytes[3] = Y_LOWER_CAL;
transferBytes[4] = X_UPPER_CAL;
transferBytes[5] = Y_UPPER_CAL;
transferBytes[6] = BOTTOM_SPEED;
transferBytes[7] = TOPSPEED;
transferBytes[8] = X_RESOLUTION_0;
transferBytes[9] = X_RESOLUTION_1;
transferBytes[10] = Y_RESOLUTION_0;
transferBytes[11] = Y_RESOLUTION_1;
transferBytes[12] = ENABLE_MASK_0_7;
transferBytes[13] = ENABLE_MASK_8_9;
transferBytes[14] = ENABLESTATUSLINK_0;
transferBytes[15] = ENABLESTATUSLINK_1;

//Writing converted values to specified memory locations with RESTART flag
writeRandomBytes16(IQS7222D_TRACKPAD_GENERAL_SETUP, 16, transferBytes, RESTART);
Serial.println("\t\t5. Write Hardware settings");

//Converting Delta Links
transferBytes[0] = TP_DELTA0_0;
transferBytes[1] = TP_DELTA0_1;
transferBytes[2] = TP_DELTA1_0;
transferBytes[3] = TP_DELTA1_1;
transferBytes[4] = TP_DELTA2_0;
transferBytes[5] = TP_DELTA2_1;
transferBytes[6] = TP_DELTA3_0;
transferBytes[7] = TP_DELTA3_1;
transferBytes[8] = TP_DELTA4_0;
transferBytes[9] = TP_DELTA4_1;
transferBytes[10] = TP_DELTA5_0;
transferBytes[11] = TP_DELTA5_1;
transferBytes[12] = TP_DELTA6_0;
transferBytes[13] = TP_DELTA6_1;
transferBytes[14] = TP_DELTA7_0;
transferBytes[15] = TP_DELTA7_1;
transferBytes[16] = TP_DELTA8_0;
transferBytes[17] = TP_DELTA8_1;
transferBytes[18] = TP_DELTA9_0;
transferBytes[19] = TP_DELTA9_1;
transferBytes[20] = TP_DELTA10_0;
transferBytes[21] = TP_DELTA10_1;
transferBytes[22] = TP_DELTA11_0;
transferBytes[23] = TP_DELTA11_1;
writeRandomBytes16(IQS7222D_DELTA_LINK_0, 24, transferBytes, RESTART);
Serial.println("\t\t6. Write Deltas");

//Converting trackpad gesture settings
transferBytes[0] = GESTURE_EVENT_ENABLE;
transferBytes[1] = MINIMUM_GESTURE_TIME;
transferBytes[2] = MAXIMUM_TAP_TIME;
transferBytes[3] = MAXIMUM_SWIPE_TIME;
transferBytes[4] = TAP_COORDINATE_LIMIT_0;
transferBytes[5] = TAP_COORDINATE_LIMIT_1;
transferBytes[6] = SWIPE_COORDINATE_THRESHOLD_0;
transferBytes[7] = SWIPE_COORDINATE_THRESHOLD_1;
writeRandomBytes16(IQS7222D_TRACKPAD_GESTURES_0, 8, transferBytes, RESTART);
Serial.println("\t\t7. Write Gesture settings");

//Converting GPIO 0 output settings
transferBytes[0] = GPIO0_SETUP_0;
transferBytes[1] = GPIO0_SETUP_1;
transferBytes[2] = GPIO0_ENABLE_MASK_0_7;
transferBytes[3] = GPIO0_ENABLE_MASK_8_13;
transferBytes[4] = GPIO0_ENABLESTATUSLINK_0;
transferBytes[5] = GPIO0_ENABLESTATUSLINK_1;
writeRandomBytes16(IQS7222D_OUTPUT_PORT0_ENABLE_CFG, 6, transferBytes, RESTART);

//Converting GPIO 1 output settings
transferBytes[0] = GPIO1_SETUP_0;
transferBytes[1] = GPIO1_SETUP_1;
transferBytes[2] = GPIO1_ENABLE_MASK_0_7;
transferBytes[3] = GPIO1_ENABLE_MASK_8_13;
transferBytes[4] = GPIO1_ENABLESTATUSLINK_0;
transferBytes[5] = GPIO1_ENABLESTATUSLINK_1;
writeRandomBytes16(IQS7222D_OUTPUT_PORT1_ENABLE_CFG, 6, transferBytes, RESTART);

//Converting GPIO 2 output settings
transferBytes[0] = GPIO2_SETUP_0;
transferBytes[1] = GPIO2_SETUP_1;
transferBytes[2] = GPIO2_ENABLE_MASK_0_7;
transferBytes[3] = GPIO2_ENABLE_MASK_8_13;
transferBytes[4] = GPIO2_ENABLESTATUSLINK_0;
transferBytes[5] = GPIO2_ENABLESTATUSLINK_1;
writeRandomBytes16(IQS7222D_OUTPUT_PORT2_ENABLE_CFG, 6, transferBytes, RESTART);
Serial.println("\t\t8. Write GPIO settings");

//Converting system settings
transferBytes[0] = SYSTEM_CONTROL_0;
transferBytes[1] = SYSTEM_CONTROL_1;
transferBytes[2] = ATI_ERROR_TIMEOUT_0;
transferBytes[3] = ATI_ERROR_TIMEOUT_1;
transferBytes[4] = ATI_REPORT_RATE_0;
transferBytes[5] = ATI_REPORT_RATE_1;
transferBytes[6] = NORMAL_MODE_TIMEOUT_0;
transferBytes[7] = NORMAL_MODE_TIMEOUT_1;
transferBytes[8] = NORMAL_MODE_REPORT_RATE_0;
transferBytes[9] = NORMAL_MODE_REPORT_RATE_1;
transferBytes[10] = LP_MODE_TIMEOUT_0;
transferBytes[11] = LP_MODE_TIMEOUT_1;
transferBytes[12] = LP_MODE_REPORT_RATE_0;
transferBytes[13] = LP_MODE_REPORT_RATE_1;
transferBytes[14] = ULP_MODE_TIMEOUT_0;
transferBytes[15] = ULP_MODE_TIMEOUT_1;
transferBytes[16] = ULP_MODE_REPORT_RATE_0;
transferBytes[17] = ULP_MODE_REPORT_RATE_1;
transferBytes[18] = ULP_ENTRY_MASK_0;
transferBytes[19] = ULP_ENTRY_MASK_1;
transferBytes[20] = TOUCH_PROX_EVENT_EN;
transferBytes[21] = POWER_ATI_TP_EVENT_EN;
transferBytes[22] = I2CCOMMS_0;
transferBytes[23] = I2CCOMMS_1;
transferBytes[24] = GPIO_OVERRIDE_0;
transferBytes[25] = GPIO_OVERRIDE_1;
transferBytes[26] = COMMS_TIMEOUT_0;
transferBytes[27] = COMMS_TIMEOUT_1;

//Writing converted values to a specified memory location with RESTART flag
writeRandomBytes(IQS7222D_CONTROL_SETTINGS, 28, transferBytes, stopOrRestart);
Serial.println("\t\t9. Write System Control Settings");
}

/*****************************************************************************/
/*                          PRIVATE METHODS                                  */
/*****************************************************************************/

/**
 * @name    readRandomBytes
 * @brief   A method that reads a specified number of bytes from a specified
 *          address and saves it into a user-supplied array. This method is used
 *          by all other methods in this class which read data from the IQS7222D
 *          device.
 * @param   memoryAddress -> The memory address at which to start reading bytes
 *                           from. See the "iqs7222d_addresses.h" file.
 * @param   numBytes      -> The number of bytes that must be read.
 * @param   bytesArray    -> The array which will store the bytes to be read,
 *                           this array will be overwritten.
 * @param  stopOrRestart -> A boolean that specifies whether the communication
 *                           window should remain open or be closed after transfer.
 *                           False keeps it open, true closes it. Use the STOP
 *                           and RESTART definitions.
 * @retval  No value is returned, however, the user-supplied array is overwritten.
 * @note   Uses standard Arduino "Wire" library which is for I2C communication.
 *          Take note that C++ cannot return an array, therefore, the array which
 *          is passed as an argument is overwritten with the required values.
 *          Pass an array to the method by using only its name, e.g. "bytesArray",
 *          without the brackets, this passes a pointer to the array.
 */
void IQS7222D::readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  /* A simple counter to assist with loading bytes into the user-supplied array. */
  uint8_t i = 0;
  /* Select the device with the address "_deviceAddress" and start communication. */
  Wire.beginTransmission(_deviceAddress);
  /* Send a bit asking for the "memoryAddress" register. */
  Wire.write(memoryAddress);
  /* Complete the selection and communication initialization. */
  /* Restart transmission for reading that follows. */
  Wire.endTransmission(RESTART);
  /* The required device has now been selected and it has been told which
     register to send information from. */

  /* Request "numBytes" bytes from the device which has address "_deviceAddress" */
  do
  {
    Wire.requestFrom((int)_deviceAddress, (int)numBytes, (int)stopOrRestart);
    /* Wait for response, this sometimes takes a few attempts*/
  } while (Wire.available() == 0);

  /* Load the received bytes into the array until there are no more */
  while (Wire.available())
  {
    /* Load the received bytes into the user-supplied array */
    bytesArray[i] = Wire.read();
    i++;
  }

  /* Always manually close the RDY window after a STOP is sent to prevent
     writing while ready closes */
  if(stopOrRestart == STOP)
  {
    iqs7222d_deviceRDY = false;
  }
}

/**
 * @name   writeRandomBytes
 * @brief  A method that writes a specified number of bytes to a specified
 *         address, the bytes to write are supplied using an array pointer.
 *         This method is used by all other methods of this class which
 *         write data to the IQS7222D device.
 * @param  memoryAddress -> The memory address at which to start writing the
 *                          bytes to. See the "iqs7222d_addresses.h" file.
 * @param  numBytes      -> The number of bytes that must be written.
 * @param  bytesArray    -> The array which stores the bytes which will be
 *                          written to the memory location.
 * @param  stopOrRestart -> A boolean that specifies whether the communication
 *                          window should remain open or be closed of transfer.
 *                          False keeps it open, true closes it. Use the STOP
 *                          and RESTART definitions.
 * @retval No value is returned, only the IQS device registers are altered.
 * @note   Uses standard Arduino "Wire" library which is for I2C communication.
 *         Take note that a full array cannot be passed to a function in C++.
 *         Pass an array to the function by using only its name, e.g. "bytesArray",
 *         without the square brackets, this passes a pointer to the array.
 *         The values to be written must be loaded into the array before passing
 *         it to the function.
 */
void IQS7222D::writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  /* Select the device with the address "_deviceAddress" and start communication. */
  Wire.beginTransmission(_deviceAddress);
  /* Specify the memory address where the IQS7222D must start saving the data,
     as designated by the "memoryAddress" variable. */
  Wire.write(memoryAddress);
  /* Write the bytes as specified in the array to which "arrayAddress" pointer points to. */
  for (int i = 0; i < numBytes; i++)
  {
    Wire.write(bytesArray[i]);
  }
  /* End the transmission, user decides to STOP or RESTART. */
  Wire.endTransmission(stopOrRestart);

  /* Always manually close the RDY window after a STOP is sent to prevent
     writing while ready closes */
  if(stopOrRestart == STOP)
  {
    iqs7222d_deviceRDY = false;
  }
}

/**
 * @name   writeRandomBytes
 * @brief  A method that writes a specified number of bytes to a specified
 *         address, the bytes to write are supplied using an array pointer.
 *         This method is used by all other methods of this class which
 *         write data to the IQS7222D device.
 * @param  memoryAddress -> The memory address at which to start writing the
 *                          bytes to. See the "iqs7222d_addresses.h" file.
 * @param  numBytes      -> The number of bytes that must be written.
 * @param  bytesArray    -> The array which stores the bytes which will be
 *                          written to the memory location.
 * @param  stopOrRestart -> A boolean that specifies whether the communication
 *                          window should remain open or be closed of transfer.
 *                          False keeps it open, true closes it. Use the STOP
 *                          and RESTART definitions.
 * @retval No value is returned, only the IQS device registers are altered.
 * @note   Uses standard Arduino "Wire" library which is for I2C communication.
 *         Take note that a full array cannot be passed to a function in C++.
 *         Pass an array to the function by using only its name, e.g. "bytesArray",
 *         without the square brackets, this passes a pointer to the array.
 *         The values to be written must be loaded into the array before passing
 *         it to the function.
 */
void IQS7222D::writeRandomBytes16(uint16_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  /* Select the device with the address "_deviceAddress" and start communication. */
  Wire.beginTransmission(_deviceAddress);
  /* Specify the memory address where the IQS7222D must start saving the data,
     as designated by the "memoryAddress" variable. */
  uint8_t addr_h, addr_l;
  addr_h = memoryAddress >> 8;
  addr_l = memoryAddress;
  Wire.write(addr_h);
  Wire.write(addr_l);
  /* Write the bytes as specified in the array which "arrayAddress" pointer
  points to. */
  for (int i = 0; i < numBytes; i++)
  {
    Wire.write(bytesArray[i]);
  }
  /* End the transmission, user decides to STOP or RESTART. */
  Wire.endTransmission(stopOrRestart);

  /* Always manually close the RDY window after a STOP is sent to prevent
  writing while ready closes */
  if(stopOrRestart == STOP)
  {
    iqs7222d_deviceRDY = false;
  }
}

/**
  * @name   getBit
  * @brief  A method that returns the chosen bit value of the provided byte.
  * @param  data       -> byte of which a given bit value needs to be calculated.
  * @param  bit_number -> a number between 0 and 7 representing the bit in question.
  * @retval The boolean value of the specific bit requested.
  */
bool IQS7222D::getBit(uint8_t data, uint8_t bit_number)
{
  return (data & ( 1 << bit_number )) >> bit_number;
}

/**
  * @name   setBit
  * @brief  A method that returns the chosen bit value of the provided byte.
  * @param  data       -> byte of which a given bit value needs to be calculated.
  * @param  bit_number -> a number between 0 and 7 representing the bit in question.
  * @retval Returns an 8-bit unsigned integer value of the given data byte with
  *         the requested bit set.
  */
uint8_t IQS7222D::setBit(uint8_t data, uint8_t bit_number)
{
	return (data |= 1UL << bit_number);
}

/**
  * @name   clearBit
  * @brief  A method that returns the chosen bit value of the provided byte.
  * @param  data       -> byte of which a given bit value needs to be calculated.
  * @param  bit_number -> a number between 0 and 7 representing the bit in question.
  * @retval Returns an 8-bit unsigned integer value of the given data byte with
  *         the requested bit cleared.
  */
uint8_t IQS7222D::clearBit(uint8_t data, uint8_t bit_number)
{
	return (data &= ~(1UL << bit_number));
}

/**
  * @name   force_I2C_communication
  * @brief  A method which writes data 0x00 to memory address 0xFF to open a
  *         communication window on the IQS7222D.
  * @param  None.
  * @retval None.
  * @note   Uses standard Arduino "Wire" library which is for I2C communication.
  */
void IQS7222D::force_I2C_communication(void)
{
  /*Ensure RDY is HIGH at the moment*/
  if (!iqs7222d_deviceRDY)
  {
    /* Select the device with the address "DEMO_IQS7211A_ADDR" and start communication. */
    Wire.beginTransmission(_deviceAddress);

    /* Write to memory address 0xFF that will prompt the IQS7211A to open a
    communication window.*/
    Wire.write(0xFF);

    /* End the transmission, the user decides to STOP or RESTART. */
    Wire.endTransmission(STOP);
  }
}
