/**
 ******************************************************************************
 * @file    IQS7222D.h
 * @brief   This file contains the header information for an IQS7222D Arduino
 *          library. The goal of the library is to provide easy functionality for
 *          initializing and using the Azoteq IQS7222D capacitive touch device.
 * @author  Azoteq
 * @version V1.1
 * @date    2023
 ******************************************************************************
 ******************************************************************************
 * @attention  Makes use of the following standard Arduino libraries:
 *       - Arduino.h   -> included in IQS7222D.h, comes standard with Arduino
 *       - Wire.h      -> Included in IQS7222D.h, comes standard with Arduino
 *
 ******************************************************************************
 */

#ifndef IQS7222D_h
#define IQS7222D_h

/* Include Files */
#include "Arduino.h"
#include "Wire.h"
#include "./inc/iqs7222d_addresses.h"

/* Device IC version select */
/* Select the IC version below by changing the value of the define (default = 0):
0: IQS7222D IC Version 1.2          
1: IQS7222D IC Version 1.1    
 */
#define IQS7222D_IC_VERSION 0

/* Public Global Definitions */
/* For use with Wire.h library. True argument with some functions closes the
   I2C communication window. */
#define STOP true
/* For use with Wire.h library. False argument with some functions keeps
   the I2C communication window open. */
#define RESTART false

/* Device Info */
#define IQS7222D_PRODUCT_NUM                    1046

// Info Flags Byte Bits.
#define IQS7222D_ATI_ACTIVE_BIT		        0
#define IQS7222D_ATI_ERROR_BIT		        1
#define IQS7222D_DEVICE_RESET_BIT		3
#define IQS7222D_POWER_EVENT_BIT_0              4
#define IQS7222D_POWER_EVENT_BIT_1              5
#define IQS7222D_NORMAL_POWER_BIT		0b00
#define IQS7222D_LOW_POWER_BIT		        0b01
#define IQS7222D_ULP_BIT		        0b10

// Events Bits
#define IQS7222D_PROX_EVENT_BIT   		0
#define IQS7222D_TOUCH_EVENT_BIT   		1
#define IQS7222D_TRACKPAD_EVENT_BIT		2 // 10
#define IQS7222D_ATI_EVENT_BIT   		4 // 12
#define IQS7222D_POWER_EVENT_BIT   		5 // 13

// Prox Channel Bits
#define IQS7222D_CH0_PROX_BIT                   0
#define IQS7222D_CH1_PROX_BIT                   1
#define IQS7222D_CH2_PROX_BIT                   2
#define IQS7222D_CH3_PROX_BIT                   3
#define IQS7222D_CH4_PROX_BIT                   4
#define IQS7222D_CH5_PROX_BIT                   5
#define IQS7222D_CH6_PROX_BIT                   6
#define IQS7222D_CH7_PROX_BIT                   7
#define IQS7222D_CH8_PROX_BIT                   0 // 8
#define IQS7222D_CH9_PROX_BIT                   1 // 9
#define IQS7222D_CH10_PROX_BIT                  2 // 10
#define IQS7222D_CH11_PROX_BIT                  3 // 11
#define IQS7222D_CH12_PROX_BIT                  4 // 12
#define IQS7222D_CH13_PROX_BIT                  5 // 13

// Touch Channel Bits
#define IQS7222D_CH0_TOUCH_BIT                  0
#define IQS7222D_CH1_TOUCH_BIT                  1
#define IQS7222D_CH2_TOUCH_BIT                  2
#define IQS7222D_CH3_TOUCH_BIT                  3
#define IQS7222D_CH4_TOUCH_BIT                  4
#define IQS7222D_CH5_TOUCH_BIT                  5
#define IQS7222D_CH6_TOUCH_BIT                  6
#define IQS7222D_CH7_TOUCH_BIT                  7
#define IQS7222D_CH8_TOUCH_BIT                  0       // 8
#define IQS7222D_CH9_TOUCH_BIT                  1       // 9
#define IQS7222D_CH10_TOUCH_BIT                 2       // 10
#define IQS7222D_CH11_TOUCH_BIT                 3       // 11
#define IQS7222D_CH12_TOUCH_BIT                 4       // 12
#define IQS7222D_CH13_TOUCH_BIT                 5       // 13

// Utility Bits
#define IQS7222D_ACK_RESET_BIT		        0
#define IQS7222D_SW_RESET_BIT		        1
#define IQS7222D_RE_ATI_BIT		        2
#define IQS7222D_RESEED_BIT		        3

#define IQS7222D_EVENT_MODE_BIT_0               6
#define IQS7222D_EVENT_MODE_BIT_1               7

// Gestures Bit
#define IQS7222D_GESTURE_TAP_BIT	        0
#define IQS7222D_GESTURE_FLICK_BIT	        1
#define IQS7222D_GESTURE_SWIPE_X_POS_BIT        2
#define IQS7222D_GESTURE_SWIPE_Y_POS_BIT        3
#define IQS7222D_GESTURE_SWIPE_X_NEG_BIT        4
#define IQS7222D_GESTURE_SWIPE_Y_NEG_BIT        5

typedef enum {
        IQS7222D_CH0 = (uint8_t) 0x00,
        IQS7222D_CH1,
        IQS7222D_CH2,
        IQS7222D_CH3,
        IQS7222D_CH4,
        IQS7222D_CH5,
        IQS7222D_CH6,
        IQS7222D_CH7,
        IQS7222D_CH8,
        IQS7222D_CH9,
        IQS7222D_CH10,
        IQS7222D_CH11,
        IQS7222D_CH12,
        IQS7222D_CH13
} iqs7222d_channel_e;
typedef enum
{
        IQS7222D_CH_NONE = (uint8_t) 0x00,
        IQS7222D_CH_PROX,
        IQS7222D_CH_TOUCH,
        IQS7222D_CH_UNKNOWN,
} iqs7222d_ch_states;

typedef enum
{
        IQS7222D_X_AXIS = (uint8_t) 0x00,
        IQS7222D_Y_AXIS
} iqs7222d_trackpad_axis_e;

typedef enum
{
        IQS7222D_GESTURE_NONE = (uint8_t) 0x00,
        IQS7222D_GESTURE_TAP,
        IQS7222D_GESTURE_FLICK,
        IQS7222D_GESTURE_SWIPE_X_POS,
        IQS7222D_GESTURE_SWIPE_Y_POS,
        IQS7222D_GESTURE_SWIPE_X_NEG,
        IQS7222D_GESTURE_SWIPE_Y_NEG
} iqs7222d_gestures_e;

typedef enum
{
        IQS7222D_NORMAL_POWER = (uint8_t) 0x00,
        IQS7222D_LOW_POWER,
        IQS7222D_ULP,
        IQS7222D_POWER_UNKNOWN
} iqs7222d_power_modes;

/* Defines and structs for IQS7222D states */
/**
 * @brief  IQS7222D Init Enumeration.
 */
typedef enum
{
        IQS7222D_INIT_NONE = (uint8_t) 0x00,
        IQS7222D_INIT_VERIFY_PRODUCT,
        IQS7222D_INIT_READ_RESET,
        IQS7222D_INIT_CHIP_RESET,
        IQS7222D_INIT_UPDATE_SETTINGS,
        IQS7222D_INIT_CHECK_RESET,
        IQS7222D_INIT_ACK_RESET,
        IQS7222D_INIT_ATI,
        IQS7222D_INIT_WAIT_FOR_ATI,
        IQS7222D_INIT_READ_DATA,
        IQS7222D_INIT_ACTIVATE_EVENT_MODE,
        IQS7222D_INIT_ACTIVATE_STREAMINTOUCH_MODE,
        IQS7222D_INIT_DONE
} iqs7222d_init_e;

typedef enum {
        IQS7222D_STATE_NONE = (uint8_t) 0x00,
        IQS7222D_STATE_START,
        IQS7222D_STATE_INIT,
        IQS7222D_STATE_SW_RESET,
        IQS7222D_STATE_CHECK_RESET,
	IQS7222D_STATE_RUN,
} iqs7222d_state_e;

/* IQS7222D Memory map data variables, only save the data that might be used
   during program runtime */
#pragma pack(1)
typedef struct
{
	/* READ ONLY */			  //    I2C Addresses:
	uint8_t VERSION_DETAILS[20]; 	  // 	0x00 -> 0x09
	uint8_t SYSTEM_STATUS[2];         // 	0x10
	uint8_t EVENTS[2]; 		  // 	0x11
	uint8_t PROX_EVENT_STATES[2]; 	  // 	0x12
	uint8_t TOUCH_EVENT_STATES[2]; 	  // 	0x13
	uint8_t TRACKPAD_X_OUTPUT[2]; 	  // 	0x14
	uint8_t TRACKPAD_Y_OUTPUT[2]; 	  // 	0x15
        uint8_t TRACKPAD_EVENT_FLAGS[2];  // 	0x16

	/* READ WRITE */		  //    I2C Addresses:
	uint8_t SYSTEM_CONTROL[2]; 	  // 	0xC0
} IQS7222D_MEMORY_MAP;
#pragma pack(4)

#pragma pack(1)
typedef struct {
        iqs7222d_state_e        state;
        iqs7222d_init_e         init_state;
}iqs7222d_s;
#pragma pack(4)

/* Class Prototype */
class IQS7222D
{
public:
        /* Public Constructors */
        IQS7222D();

        /* Public Device States */
        iqs7222d_s iqs7222d_state;

        /* Public Variables */
        IQS7222D_MEMORY_MAP IQSMemoryMap;
        bool new_data_available;

        /* Public Methods */
        void begin(uint8_t deviceAddressIn, uint8_t readyPinIn);
        bool init(void);
        void run(void);
        void queueValueUpdates(void);
        bool readATIactive(void);
        uint16_t getProductNum(bool stopOrRestart);
        uint8_t getMajorVersion(bool stopOrRestart);
        uint8_t getMinorVersion(bool stopOrRestart);
        void acknowledgeReset(bool stopOrRestart);
        void ReATI(bool stopOrRestart);
        void SW_Reset(bool stopOrRestart);
        void writeMM(bool stopOrRestart);
        void clearRDY(void);
        bool getRDYStatus(void);

        void setStreamMode(bool stopOrRestart);
        void setEventMode(bool stopOrRestart);
        void setStreamInTouchMode(bool stopOrRestart);

        void updateInfoFlags(bool stopOrRestart);
        uint16_t getTrackPadCoordinate(iqs7222d_trackpad_axis_e TrackPadAxisSelect);
        iqs7222d_gestures_e getGestures(void);
        bool getTrackpadEvent(void);
        iqs7222d_power_modes getPowerMode(void);

        bool checkReset(void);

        bool channel_touchState(iqs7222d_channel_e channel);
        bool getTouchEvent(void);
        bool channel_proxState(iqs7222d_channel_e channel);
        bool getProxEvent(void);

        void force_I2C_communication(void);

private:
        /* Private Variables */
        uint8_t _deviceAddress;

        /* Private Methods */
        void readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
        void writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
        void writeRandomBytes16(uint16_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
        bool getBit(uint8_t data, uint8_t bit_number);
        uint8_t setBit(uint8_t data, uint8_t bit_number);
        uint8_t clearBit(uint8_t data, uint8_t bit_number);
};
#endif /* IQS7222D_h */
